@@ -1,8 +1,7 @@
 # Author:    Lisandro Dalcin
 # Contact:   dalcinl@gmail.com
-# Revision:  $Revision$
-# Date:      $Date$
 # Copyright: This module has been placed in the public domain.
+# Id:        $Id: MPI.py 118 2007-08-01 00:42:30Z dalcinl $
 
 """
 Message Passing Interface module.
@@ -10,7 +9,7 @@ Message Passing Interface module.
 This module provides MPI support to run Python scripts in parallel
 environments. It is constructed on top of the MPI-1/MPI-2
 specification, but provides an object oriented interface which closely
-follows standard MPI-2 C++ bindings. Almost all MPI-2 features are
+follows the standard MPI-2 C++ bindings. Almost all MPI-2 features are
 available provided that the underlying MPI implementation supports
 them.
 
@@ -22,9 +21,6 @@ communications, as well as parallel file I/O (read, write)
 operations. Group and communicator (inter, intra and topologies)
 management is fully supported, as well as creation of user-defined
 datatypes.
-
-.. note:: It is recommendable to use this module with a parallelized
-   version of the Python interpreter.
 """
 
 __docformat__ = 'reStructuredText'
@@ -36,9 +32,8 @@ __docformat__ = 'reStructuredText'
 
 __author__    = 'Lisandro Dalcin'
 __credits__   = 'MPI Forum, MPICH Team, Open MPI Team.'
-__date__      = '1 Jun 2007'
-__version__   = '0.4.0'
-__revision__  = '$Id: MPI.py 34 2007-06-01 20:26:10Z dalcinl $'
+__version__   = '0.5.0'
+__revision__  = '$Id: MPI.py 118 2007-08-01 00:42:30Z dalcinl $'
 
 
 # --------------------------------------------------------------------
@@ -168,7 +163,11 @@ def Buffer(*args):
         else:
             raise TypeError('Buffer() takes at most 3 arguments '
                             '(%d given))' % (len(args) + 1))
-    assert isinstance(count, (int, long)) or count is None
+    try:
+        _ints = (int, long)
+    except NameError:
+        _ints = (int, )
+    assert isinstance(count, _ints) or count is None
     assert isinstance(datatype, Datatype)
     return (data, count, datatype)
 
@@ -453,6 +452,7 @@ SHORT           = Datatype(_mpi.SHORT)
 UNSIGNED_SHORT  = Datatype(_mpi.UNSIGNED_SHORT)
 INT             = Datatype(_mpi.INT)
 UNSIGNED        = Datatype(_mpi.UNSIGNED)
+UNSIGNED_INT    = UNSIGNED
 LONG            = Datatype(_mpi.LONG)
 UNSIGNED_LONG   = Datatype(_mpi.UNSIGNED_LONG)
 FLOAT           = Datatype(_mpi.FLOAT)
@@ -463,18 +463,18 @@ PACKED          = Datatype(_mpi.PACKED)
 # Datatypes for reduction operations
 SHORT_INT       = Datatype(_mpi.SHORT_INT)
 TWOINT          = Datatype(_mpi.TWOINT)
-INT_INT         = Datatype(_mpi.INT_INT)
+INT_INT         = TWOINT
 LONG_INT        = Datatype(_mpi.LONG_INT)
 FLOAT_INT       = Datatype(_mpi.FLOAT_INT)
 DOUBLE_INT      = Datatype(_mpi.DOUBLE_INT)
 LONG_DOUBLE_INT = Datatype(_mpi.LONG_DOUBLE_INT)
 # Optional datatypes
 LONG_LONG          = Datatype(_mpi.LONG_LONG)
-LONG_LONG_INT      = Datatype(_mpi.LONG_LONG_INT)
 UNSIGNED_LONG_LONG = Datatype(_mpi.UNSIGNED_LONG_LONG)
+LONG_LONG_INT      = Datatype(_mpi.LONG_LONG_INT)
 # Special datatypes (for constructing derived datatypes)
-UB = Datatype(_mpi.UB)
-LB = Datatype(_mpi.LB)
+UB = Datatype(_mpi.UB) #: upper-bound marker (deprecated in MPI-2)
+LB = Datatype(_mpi.LB) #: lower-bound marker (deprecated in MPI-2)
 
 
 
@@ -497,13 +497,13 @@ def Get_address(location):
 
 def Alloc_mem(size, info=None):
     """
-    Allocate memory for message passing and RMA
+    Allocate memory for message passing and RMA.
     """
     return _mpi.alloc_mem(size, info)
 
 def Free_mem(base):
     """
-    Free memory allocated with 'Alloc_mem()'
+    Free memory allocated with `Alloc_mem()`.
     """
     return _mpi.free_mem(base)
 
@@ -781,7 +781,7 @@ class Op(_mpi.Op):
         """
         Create a user-defined combination function handle.
         """
-        if callable(function):
+        if hasattr(function, '__call__'):
             self.__function = function
             self.__commute  = bool(commute)
         else:
@@ -802,25 +802,24 @@ class Op(_mpi.Op):
 
 OP_NULL = Op(_mpi.OP_NULL)
 """Null operation handle"""
-MAX     = Op(_mpi.MAX,     _op.MAX,     True)
-MIN     = Op(_mpi.MIN,     _op.MIN,     True)
-SUM     = Op(_mpi.SUM,     _op.SUM,     True)
-PROD    = Op(_mpi.PROD,    _op.PROD,    True)
-LAND    = Op(_mpi.LAND,    _op.LAND,    True)
-BAND    = Op(_mpi.BAND,    _op.BAND,    True)
-LOR     = Op(_mpi.LOR,     _op.LOR,     True)
-BOR     = Op(_mpi.BOR,     _op.BOR,     True)
-LXOR    = Op(_mpi.LXOR,    _op.LXOR,    True)
-BXOR    = Op(_mpi.BXOR,    _op.BXOR,    True)
-MAXLOC  = Op(_mpi.MAXLOC,  _op.MAXLOC,  True)
-MINLOC  = Op(_mpi.MINLOC,  _op.MINLOC,  True)
-REPLACE = Op(_mpi.REPLACE, _op.REPLACE, False)
-"""Special operation for one-sided communications"""
+MAX     = Op(_mpi.MAX,     _op.MAX,     True)  #: Maximum
+MIN     = Op(_mpi.MIN,     _op.MIN,     True)  #: Minimum
+SUM     = Op(_mpi.SUM,     _op.SUM,     True)  #: Sum
+PROD    = Op(_mpi.PROD,    _op.PROD,    True)  #: Product
+LAND    = Op(_mpi.LAND,    _op.LAND,    True)  #: Logical and
+BAND    = Op(_mpi.BAND,    _op.BAND,    True)  #: Bit-wise and
+LOR     = Op(_mpi.LOR,     _op.LOR,     True)  #: Logical or
+BOR     = Op(_mpi.BOR,     _op.BOR,     True)  #: Bit-wise or
+LXOR    = Op(_mpi.LXOR,    _op.LXOR,    True)  #: Logical xor
+BXOR    = Op(_mpi.BXOR,    _op.BXOR,    True)  #: Bit-wise xor
+MAXLOC  = Op(_mpi.MAXLOC,  _op.MAXLOC,  True)  #: Maximum and location
+MINLOC  = Op(_mpi.MINLOC,  _op.MINLOC,  True)  #: Minimum and location
+REPLACE = Op(_mpi.REPLACE, _op.REPLACE, False) #: Replace (for RMA)
 
 
 
 # --------------------------------------------------------------------
-# Info
+# [4.10] The Info Object
 # --------------------------------------------------------------------
 
 class Info(_mpi.Info):
@@ -864,41 +863,47 @@ class Info(_mpi.Info):
         Add the (key,value) pair to info, and overrides the value if a
         value for the same key was previously set.
 
-        .. note:: If either key or value are larger than the allowed
-           maximums, the errors `ERR_INFO_KEY` or
+        .. note:: If either `key` or `value` are larger than the
+           allowed maximums, the errors `ERR_INFO_KEY` or
            `ERR_INFO_VALUE` are raised, respectively.
         """
         _mpi.info_set(self, key, value)
 
     def Delete(self, key):
         """
-        Delete a (key,value) pair from info.
+        Remove a (key,value) pair from info.
 
-        .. note:: If key is not defined in info, a error of class
+        .. note:: If `key` is not present, a error of class
           `ERR_INFO_NOKEY` is raised.
         """
         _mpi.info_delete(self, key)
 
+    Del = Delete ##: convenience alias for `Info.Delete()`
+
     def Get(self, key, maxlen=-1):
         """
-        Retrieves the value associated with a key.
+        Retrieve the value associated with a key.
+
+        .. note:: If `key` is not present, ``None`` is returned.
         """
         return _mpi.info_get(self, key, maxlen)
 
     def Get_nkeys(self):
         """
-        Returns the number of currently defined keys in info.
+        Return the number of currently defined keys in info.
         """
         return _mpi.info_get_nkeys(self)
     
     def Get_nthkey(self, n):
         """
-        Returns the nth defined key in info. Keys are numbered in the
+        Return the nth defined key in info. Keys are numbered in the
         range [0, N) where N is the value returned by
-        `Info.Get_nkeys()`. All keys between 0 and N-1 are guaranteed
-        to be defined. The number of a given key does not change as
-        long as info is not modified with `Info.Set()` or
-        `Info.Delete()`.
+        `Info.Get_nkeys()`.
+
+        .. note:: All keys between 0 and N-1 are guaranteed to be
+           defined. The number of a given key does not change as long
+           as info is not modified with `Info.Set()` or
+           `Info.Delete()`.
         """
         return _mpi.info_get_nthkey(self, n)
 
@@ -906,32 +911,35 @@ class Info(_mpi.Info):
     # ---------------
     
     def __len__(self):
-        if self == _mpi.INFO_NULL:
-            return 0
-        else:
-            return _mpi.info_get_nkeys(self)
+        if not self: return 0
+        return _mpi.info_get_nkeys(self)
 
     def __getitem__(self, key):
+        if not self: raise KeyError('info is INFO_NULL')
         value, flag = _mpi.info_get(self, key)
-        if not flag:
-            raise KeyError(key)
+        if not flag: raise KeyError(key)
         return value
 
     def __setitem__(self, key, value):
+        if not self: raise KeyError('info is INFO_NULL')
         return _mpi.info_set(self, key, value)
 
     def __delitem__(self, key):
-        _mpi.info_delete(self, key)
+        if not self: raise KeyError('info is INFO_NULL')
+        return _mpi.info_delete(self, key)
 
     def __contains__(self, key):
+        if not self: return False
         return _mpi.info_get_valuelen(self, key)[1]
 
     def __iter__(self):
-        if self == _mpi.INFO_NULL:
-            return
+        if not self: return
         nkeys = _mpi.info_get_nkeys(self)
-        for nthkey in xrange(nkeys):
-            yield _mpi.info_get_nthkey(self, nthkey)
+        nthkey = _mpi.info_get_nthkey
+        try:    iterkeys = xrange
+        except: iterkeys = range
+        for k in iterkeys(nkeys):
+            yield nthkey(self, k)
 
 
 # Predefined info handle
@@ -1159,7 +1167,7 @@ class Comm(_mpi.Comm):
     
     def Clone(self):
         """
-        Duplicate an existing communicator..
+        Duplicate an existing communicator.
         """
         newcomm = _mpi.comm_dup(self)
         return type(self)(newcomm)
@@ -1187,7 +1195,7 @@ class Comm(_mpi.Comm):
         Blocking send.
 
         .. note:: This function may block until the message is
-           received. Whether or not ``MPI_SEND`` blocks depends on
+           received. Whether or not `Send` blocks depends on
            several factors and is implementation dependent.
         """
         buf, fastmode = _mpi.make_buf(buf)
@@ -1195,11 +1203,13 @@ class Comm(_mpi.Comm):
             _mpi.send(buf, dest, tag, self)
             return None
         else:
+            serializer = self.SERIALIZER
+            send = _mpi.send_pickled
             if dest != _mpi.PROC_NULL:
-                buf = self.SERIALIZER.dump(buf)
+                buf = serializer.dump(buf)
             else:
                 buf = None
-            _mpi.send_string(buf, dest, tag, self)
+            send(buf, dest, tag, self)
             return None
     
     def Recv(self, buf=None, source=0, tag=0, status=None):
@@ -1213,9 +1223,11 @@ class Comm(_mpi.Comm):
             _mpi.recv(buf, source, tag, self, status)
             return None
         else:
-            buf = _mpi.recv_string(buf, source, tag, self, status)
+            serializer = self.SERIALIZER
+            recv = _mpi.recv_pickled
+            buf = recv(buf, source, tag, self, status)
             if source != _mpi.PROC_NULL:
-                buf = self.SERIALIZER.load(buf)
+                buf = serializer.load(buf)
             return buf
         
     # Send-Receive
@@ -1244,13 +1256,14 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            sendrecv = _mpi.sendrecv_pickled
             if dest != _mpi.PROC_NULL:
                 sendbuf = serializer.dump(sendbuf)
             else:
                 sendbuf = None
-            recvbuf = _mpi.sendrecv_string(sendbuf, dest, sendtag,
-                                           recvbuf, source, recvtag,
-                                           self, status)
+            recvbuf = sendrecv(sendbuf, dest, sendtag,
+                               recvbuf, source, recvtag,
+                               self, status)
             if recvbuf is not None:
                 recvbuf = serializer.load(recvbuf)
             return recvbuf
@@ -1422,6 +1435,7 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            bcast = _mpi.bcast_pickled
             if _mpi.comm_test_inter(self):
                 if root == _mpi.ROOT:
                     buf = serializer.dump(buf)
@@ -1432,7 +1446,7 @@ class Comm(_mpi.Comm):
                     buf = serializer.dump(buf)
                 else:
                     buf = None
-            buf = _mpi.bcast_string(buf, root, self)
+            buf = bcast(buf, root, self)
             if buf is not None:
                 buf = serializer.load(buf)
             return buf
@@ -1448,6 +1462,7 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            gather = _mpi.gather_pickled
             if _mpi.comm_test_inter(self):
                 if root != _mpi.ROOT and root != _mpi.PROC_NULL:
                     sendbuf = serializer.dump(sendbuf)
@@ -1455,9 +1470,10 @@ class Comm(_mpi.Comm):
                     sendbuf = None
             else:
                 sendbuf = serializer.dump(sendbuf)
-            recvbuf = _mpi.gather_string(sendbuf, recvbuf, root, self)
+            recvbuf = gather(sendbuf, recvbuf, root, self)
             if recvbuf is not None:
                 recvbuf = map(serializer.load, recvbuf)
+                recvbuf = list(recvbuf)
             return recvbuf
 
     def Scatter(self, sendbuf=None, recvbuf=None, root=0):
@@ -1472,17 +1488,20 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            scatter = _mpi.scatter_pickled
             if _mpi.comm_test_inter(self):
                 if root == _mpi.ROOT:
                     sendbuf = map(serializer.dump, sendbuf)
+                    sendbuf = list(sendbuf)
                 else:
                     sendbuf = None
             else:
                 if root == _mpi.comm_rank(self):
                     sendbuf = map(serializer.dump, sendbuf)
+                    sendbuf = list(sendbuf)
                 else:
                     sendbuf = None
-            recvbuf = _mpi.scatter_string(sendbuf, recvbuf, root, self)
+            recvbuf = scatter(sendbuf, recvbuf, root, self)
             if recvbuf is not None:
                 recvbuf = serializer.load(recvbuf)
             return recvbuf
@@ -1499,9 +1518,11 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            allgather = _mpi.allgather_pickled
             sendbuf = serializer.dump(sendbuf)
-            recvbuf = _mpi.allgather_string(sendbuf, recvbuf, self)
+            recvbuf = allgather(sendbuf, recvbuf, self)
             recvbuf = map(serializer.load, recvbuf)
+            recvbuf = list(recvbuf)
             return recvbuf
 
     def Alltoall(self, sendbuf, recvbuf=None):
@@ -1516,9 +1537,12 @@ class Comm(_mpi.Comm):
             return None
         else:
             serializer = self.SERIALIZER
+            alltoall = _mpi.alltoall_pickled
             sendbuf = map(serializer.dump, sendbuf)
-            recvbuf = _mpi.alltoall_string(sendbuf, recvbuf, self)
+            sendbuf = list(sendbuf)
+            recvbuf = alltoall(sendbuf, recvbuf, self)
             recvbuf = map(serializer.load, recvbuf)
+            recvbuf = list(recvbuf)
             return recvbuf
 
     # Global Reduction Operations
@@ -1538,8 +1562,8 @@ class Comm(_mpi.Comm):
             recvbuf = self.Gather(sendbuf, None, root)
             if recvbuf is not None:
                 if op in (MAXLOC, MINLOC):
-                    recvbuf = zip(recvbuf, xrange(len(recvbuf)))
-                recvbuf = reduce(op, recvbuf)
+                    recvbuf = zip(recvbuf, range(len(recvbuf)))
+                recvbuf = _mpi._reduce(op, recvbuf)
             return recvbuf
 
     def Allreduce(self, sendbuf, recvbuf=None, op=SUM):
@@ -1555,11 +1579,12 @@ class Comm(_mpi.Comm):
             # naive implementation
             serializer = self.SERIALIZER
             sendbuf = serializer.dump(sendbuf)
-            recvbuf = _mpi.allgather_string(sendbuf, recvbuf, self)
+            recvbuf = _mpi.allgather_pickled(sendbuf, recvbuf, self)
             recvbuf = map(serializer.load, recvbuf)
+            recvbuf = list(recvbuf)
             if op in (MAXLOC, MINLOC):
-                recvbuf = zip(recvbuf, xrange(len(recvbuf)))
-            recvbuf = reduce(op, recvbuf)
+                recvbuf = zip(recvbuf, range(len(recvbuf)))
+            recvbuf = _mpi._reduce(op, recvbuf)
             return recvbuf
     
     ## def Reduce_scatter(self, buff, op=SUM):
@@ -1671,7 +1696,7 @@ class Comm(_mpi.Comm):
         """Port (Communicator nested class)."""
         TAG = _mpi.TAG_UB-1
         def __init__(self, comm, pid):
-            if pid not in xrange(len(comm)):
+            if pid < 0 or pid >= len(comm):
                 raise ValueError('port number out of range')
             self._comm = comm
             self._pid  = pid
@@ -1792,24 +1817,24 @@ class Intracomm(Comm):
         Duplicate intracommunicator.
         """
         newcomm = _mpi.comm_dup(self)
-        return Intracomm(newcomm)
+        return type(self)(newcomm)
     
     def Create(self, group):
         """
         Create intracommunicator from group.
         """
         newcomm = _mpi.comm_create(self, group)
-        return Intracomm(newcomm)
+        return type(self)(newcomm)
     
     def Split(self, color, key=0):
         """
-        Split communicator by color and key.
+        Split intracommunicator by color and key.
         """
         newcomm = _mpi.comm_split(self, color, key)
-        return Intracomm(newcomm)
+        return type(self)(newcomm)
 
     def Create_intercomm(self, local_leader,
-                         peer_comm, remote_leader,
+                         peer_comm, remote_leader=0,
                          tag=0):
         """
         Create intercommunicator.
@@ -1855,8 +1880,9 @@ class Intracomm(Comm):
             recvbuf = self.Gather(sendbuf, root=0)
             if _mpi.comm_rank(self) == 0:
                 if op in (MAXLOC, MINLOC):
-                    recvbuf = zip(recvbuf, xrange(len(recvbuf)))
-                for i in xrange(1, len(recvbuf)):
+                    recvbuf = zip(recvbuf, range(len(recvbuf)))
+                    recvbuf = list(recvbuf)
+                for i in range(1, len(recvbuf)):
                     recvbuf[i] = op(recvbuf[i-1], recvbuf[i])
             recvbuf = self.Scatter(recvbuf, root=0)
             return recvbuf
@@ -1877,8 +1903,9 @@ class Intracomm(Comm):
             recvbuf = self.Gather(sendbuf, root=0)
             if _mpi.comm_rank(self) == 0:
                 if op in (MAXLOC, MINLOC):
-                    recvbuf = zip(recvbuf, xrange(len(recvbuf)))
-                for i in xrange(1, len(recvbuf)):
+                    recvbuf = zip(recvbuf, range(len(recvbuf)))
+                    recvbuf = list(recvbuf)
+                for i in range(1, len(recvbuf)):
                     recvbuf[i] = op(recvbuf[i-1], recvbuf[i])
                 recvbuf.insert(0, None)
                 recvbuf.pop(-1)
@@ -1890,11 +1917,13 @@ class Intracomm(Comm):
         
     # Starting Processes
     
-    def Spawn(self, command, args=None, maxprocs=1, info=None, root=0):
+    def Spawn(self, command, args, maxprocs,
+              info=None, root=0, errcodes=None):
         """
         Spawn instances of a single MPI application.
         """
-        newcomm = _mpi.comm_spawn(command, args, maxprocs, info, root, self)
+        newcomm = _mpi.comm_spawn(command, args, maxprocs,
+                                  info, root, self, errcodes)
         return Intercomm(newcomm)
 
     # Server Routines
@@ -1919,11 +1948,16 @@ class Intracomm(Comm):
 # Predefined communicator handles
 # -------------------------------
 
-COMM_SELF  = SELF  =  Intracomm(_mpi.COMM_SELF_DUP)
+COMM_SELF = Intracomm(_mpi.COMM_SELF_DUP)
 """Duplication of ``MPI_COMM_SELF`` handle"""
-COMM_WORLD = WORLD =  Intracomm(_mpi.COMM_WORLD_DUP)
+COMM_WORLD = Intracomm(_mpi.COMM_WORLD_DUP)
 """Duplication of ``MPI_COMM_WORLD`` handle"""
 
+SELF = COMM_SELF
+"""Convenience alias for `COMM_SELF`"""
+WORLD = COMM_WORLD
+"""Convenience alias for `COMM_WORLD`"""
+
 __COMM_SELF__  = Intracomm(_mpi.COMM_SELF)
 """Actual handle to ``MPI_COMM_SELF``"""
 __COMM_WORLD__ = Intracomm(_mpi.COMM_WORLD)
@@ -1966,7 +2000,7 @@ class Cartcomm(Intracomm):
         Duplicate cartesian communicator.
         """
         newcomm = _mpi.comm_dup(self)
-        return Cartcomm(newcomm)
+        return type(self)(newcomm)
 
     # Cartesian Inquiry Functions
     # ---------------------------
@@ -2017,7 +2051,7 @@ class Cartcomm(Intracomm):
         subgrids.
         """
         newcomm = _mpi.cart_sub(self, remain_dims)
-        return Cartcomm(newcomm)
+        return type(self)(newcomm)
 
     # Cartesian Low-Level Functions
     # -----------------------------
@@ -2060,7 +2094,7 @@ class Graphcomm(Intracomm):
         Duplicate graph communicator.
         """
         newcomm = _mpi.comm_dup(self)
-        return Graphcomm(newcomm)
+        return type(self)(newcomm)
 
     # Graph Inquiry Functions
     # -----------------------
@@ -2155,21 +2189,21 @@ class Intercomm(Comm):
         Duplicate intercommunicator.
         """
         newcomm = _mpi.comm_dup(self)
-        return Intercomm(newcomm)
+        return type(self)(newcomm)
     
     def Create(self, group):
         """
         Create intercommunicator from group.
         """
         newcomm = _mpi.comm_create(self, group)
-        return Intercomm(newcomm)
+        return type(self)(newcomm)
     
     def Split(self, color=0, key=0):
         """
-        Split communicator by color and key.
+        Split intercommunicator by color and key.
         """
         newcomm = _mpi.comm_split(self, color, key)
-        return Intercomm(newcomm)
+        return type(self)(newcomm)
 
     def Merge(self, high=False):
         """
@@ -2187,11 +2221,11 @@ class Intercomm(Comm):
 
 
 # --------------------------------------------------------------------
-# Process Creation and Management
+# [5] Process Creation and Management
 # --------------------------------------------------------------------
 
-# Server Routines
-# ---------------
+# [5.4.2] Server Routines
+# -----------------------
 
 def Open_port(info=None):
     """
@@ -2206,8 +2240,8 @@ def Close_port(port_name):
     """
     return _mpi.close_port(port_name)
 
-# Service Publishing
-# ------------------
+# [5.4.4] Name Publishing
+# -----------------------
 
 def Publish_name(service_name, info, port_name):
     """
@@ -2261,11 +2295,12 @@ class Win(_mpi.Win):
     # -----------------------
 
     ## @classmethod
-    def Create(cls, memory, disp, info, comm):
+    def Create(cls, memory, disp_unit=1, info=None, comm=None):
         """
         Create an window object for one-sided communication.
         """
-        win = _mpi.win_create(memory, disp, info, comm)
+        assert comm is not None
+        win = _mpi.win_create(memory, disp_unit, info, comm)
         return cls(win)
 
     Create = classmethod(Create)
@@ -2279,24 +2314,6 @@ class Win(_mpi.Win):
     # [6.2.2] Window Attributes
     # -------------------------
 
-    def Get_base(self):
-        """
-        Get window base address (attribute ``MPI_WIN_BASE``).
-        """
-        return _mpi.win_get_base(self)
-    
-    def Get_size(self):
-        """
-        Get window size, in bytes (attribute ``MPI_WIN_SIZE``).
-        """
-        return _mpi.win_get_size(self)
-
-    def Get_disp_unit(self):
-        """
-        Get displacement unit (attribute ``MPI_WIN_DISP_UNIT``).
-        """
-        return _mpi.win_get_disp(self)
-    
     def Get_group(self): 
         """
         Return a duplicate of the group of the
@@ -2311,41 +2328,31 @@ class Win(_mpi.Win):
     # [6.3.1] Put
     # -----------
     
-    def Put(self, origin_buf, target_rank,
-            target_disp=0, target_count=-1, target_datatype=None):
+    def Put(self, origin, target_rank, target=None):
         """
         Put data into a memory window on a remote process.
         """
-        return _mpi.win_put(origin_buf, target_rank,
-                            target_disp, target_count, target_datatype,
-                            self)
+        return _mpi.win_put(origin, target_rank, target, self)
 
     # [6.3.2] Get
     # -----------
     
-    def Get(self, origin_buf, target_rank,
-            target_disp=0, target_count=-1, target_datatype=None):
+    def Get(self, origin, target_rank, target=None):
         """
         Get data from a memory window on a remote process.
         """
-        return _mpi.win_get(origin_buf, target_rank,
-                            target_disp, target_count, target_datatype,
-                            self)
+        return _mpi.win_get(origin, target_rank, target, self)
 
     # [6.3.4] Accumulate Functions
     # ----------------------------
     
-    def Accumulate(self, origin_buf, target_rank,
-                   target_disp=0, target_count=-1,
-                   target_datatype=None, op=SUM):
+    def Accumulate(self, origin, target_rank, target=None, op=SUM):
         """
-        Accumulate data into the target process using remote memory
-        access.
+        Accumulate data into the target process
+        using remote memory access.
         """
-        return _mpi.win_accumulate(origin_buf, target_rank,
-                                   target_disp, target_count,
-                                   target_datatype,
-                                   op, self)
+        return _mpi.win_accumulate(origin, target_rank,
+                                   target, op, self)
     
     # [6.4] Synchronization Calls
     # ---------------------------
@@ -2412,14 +2419,14 @@ class Win(_mpi.Win):
     
     def Get_errhandler(self):
         """
-        Get the error handler for a communicator.
+        Get the error handler for a window.
         """
         eh = _mpi.win_get_errhandler(self)
         return Errhandler(eh)
 
     def Set_errhandler(self, errhandler):
         """
-        Set the error handler for a communicator.
+        Set the error handler for a window.
         """
         _mpi.win_set_errhandler(self, errhandler)
 
@@ -2428,31 +2435,32 @@ class Win(_mpi.Win):
     
     def Get_name(self):
         """
-        Get the print name from this window.
+        Get the print name from a window.
         """
         return _mpi.win_get_name(self)
 
     def Set_name(self, name):
         """
-        Set the print name for this window.
+        Set the print name for a window.
         """
         return _mpi.win_set_name(self, name)
 
+    
     # Properties
     # ----------
 
-    base      = property(_mpi.win_get_base,
-                         doc="window base address")
-    size      = property(_mpi.win_get_size,
-                         doc="window size, in bytes")
-    disp_unit = property(_mpi.win_get_disp,
-                         doc="displacement unit")
-
+    attrs  = property(_mpi.win_get_attrs,
+                      doc="window attributes (base, size, disp_unit)")
+    
+    memory = property(_mpi.win_get_memory,
+                      doc="window memory buffer")
+    
     name = property(_mpi.win_get_name,
                     _mpi.win_set_name,
                     doc="window name")
 
 
+
 # Predefined window handle
 
 WIN_NULL = Win(_mpi.WIN_NULL)
@@ -2793,7 +2801,7 @@ class File(_mpi.File):
     ## _mpi.file_{read|write}_split(..., A, B) ->
     ## A: all(1)/ordered(3), B: begin(0)/end(1)
 
-    # explicit offset:
+    # explicit offset
 
     def Read_at_all_begin(self, offset, buf):
         """
@@ -3006,13 +3014,13 @@ def Is_finalized():
     return _mpi.finalized()
 
 # Levels of MPI threading support.
-THREAD_SINGLE     = _mpi.THREAD_SINGLE
+THREAD_SINGLE = _mpi.THREAD_SINGLE
 """Only one thread will execute"""
-THREAD_FUNNELED   = _mpi.THREAD_FUNNELED
+THREAD_FUNNELED = _mpi.THREAD_FUNNELED
 """MPI calls are *funneled* to the main thread"""
 THREAD_SERIALIZED = _mpi.THREAD_SERIALIZED
 """MPI calls are *serialized*"""
-THREAD_MULTIPLE   = _mpi.THREAD_MULTIPLE
+THREAD_MULTIPLE = _mpi.THREAD_MULTIPLE
 """Multiple threads may call MPI"""
 
 def Init_thread(required):
@@ -3072,6 +3080,11 @@ IO = _mpi.IO
 """Rank of a process providing regular I/O facilities"""
 WTIME_IS_GLOBAL = _mpi.WTIME_IS_GLOBAL
 """Indicate whether clocks are synchronized"""
+UNIVERSE_SIZE = _mpi.UNIVERSE_SIZE
+"""Indicate how many processes can usefully be started
+on the underlying MPI runtime environment"""
+APPNUM = _mpi.APPNUM
+"""Indicate if a process was spawned"""
 
 def Get_processor_name():
     """
@@ -3087,16 +3100,15 @@ def Wtime():
     Return an elapsed time on the calling processor.
 
     .. note:: This is intended to be a high-resolution, elapsed (or
-       wall) clock.  Use ``MPI_WTICK`` to determine the resolution of
-       ``MPI_WIME`` . If the attribute ``MPI_WTIME_IS_GLOBAL`` is
-       defined and true, then the value is synchronized across all
-       processes in ``MPI_COMM_WORLD``.
+       wall) clock.  Use `Wtick` to determine the resolution of
+       `Wtime` . If the attribute `WTIME_IS_GLOBAL` is ``True``, then
+       the value is synchronized across all processes in `COMM_WORLD`.
     """
     return _mpi.wtime()
 
 def Wtick():
     """
-    Return the resolution of ``MPI_WTIME``.
+    Return the resolution of ``Wtime``.
     """
     return _mpi.wtick()
 
@@ -3270,6 +3282,15 @@ ERR_IO                    = _mpi.ERR_IO
 ERR_LASTCODE = _mpi.ERR_LASTCODE
 
 
+# Convenience constants 
+# ---------------------
+
+WORLD_SIZE = _mpi.WORLD_SIZE
+"""*size* of `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_size()``."""
+WORLD_RANK = _mpi.WORLD_RANK
+"""*rank* in `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_rank()``."""
+
+
 
 # --------------------------------------------------------------------
 # SWIG support
@@ -3307,33 +3328,16 @@ for klass in (Datatype, Status, Request, Op, Info,
 del SWIG, klass
 
 
-# --------------------------------------------------------------------
-# Convenience constants (some may go away in future releases)
-# --------------------------------------------------------------------
-
-WORLD_SIZE = _mpi.WORLD_SIZE
-"""*size* of `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_size()``."""
-WORLD_RANK = _mpi.WORLD_RANK
-"""*rank* of `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_rank()``."""
-size = _mpi.WORLD_SIZE
-"""*size* of `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_size()``."""
-rank = _mpi.WORLD_RANK
-"""*rank* of `COMM_WORLD`, shortcut for ``COMM_WORLD.Get_rank()``."""
-zero = (WORLD_RANK==0)
-"""``True`` if this process has *rank* 0 in ``COMM_WORLD``."""
-last = (WORLD_RANK==(WORLD_SIZE-1))
-"""``True`` if this process has *rank* *size-1* in ``COMM_WORLD``."""
-even = ((WORLD_RANK%2)!=1)
-"""``True`` if this process has even *rank* in ``COMM_WORLD``."""
-odd  = ((WORLD_RANK%2)==1)
-"""``True`` if this process has odd *rank* in ``COMM_WORLD``."""
-
-
-
 # --------------------------------------------------------------------
 # Module initialization
 # --------------------------------------------------------------------
 
+ExceptionMPI = Exception
+__all__ = [sym for sym in dir() if not sym.startswith('_')]
+__all__.remove('Exception')
+sym = None
+del sym
+
 def _mpi_init():
     """Initialize _mpi module"""
     try:
@@ -3362,7 +3366,6 @@ del atexit, _mpi_fini
 
 
 
-
 # --------------------------------------------------------------------
 # Misc utils (some may go away in future releases)
 # --------------------------------------------------------------------
@@ -3374,7 +3377,7 @@ def _mpi_info():
     """
     return _mpi._mpi_info()
 
-def distribute(N, B, i):
+def _distribute(N, B, i):
     """
     Compute interval [low, high) in bin i
     when sequence of N elements has to be
@@ -3382,26 +3385,30 @@ def distribute(N, B, i):
     """
     return _mpi.distribute(N, B, i)
 
-def pprint(message=None, comm=None, root=0):
+def _pprint(message=None, comm=None, root=0):
     """
     Synchronized print.
     """
+    import sys
     if comm is None:
         comm = COMM_WORLD
     inbuf  = comm.SERIALIZER.dump(message)
-    result = _mpi.gather_string(inbuf, None, root, comm)
+    result = _mpi.gather_pickled(inbuf, None, root, comm)
     if comm.Get_rank() == root:
         for i in result:
             msg = comm.SERIALIZER.load(i)
-            print msg
+            sys.stdout.write(str(msg)+'\n')
+            sys.stdout.flush()
     return
 
-def rprint(message=None, comm=None, root=0):
+def _rprint(message=None, comm=None, root=0):
     """
     Rooted print.
     """
+    import sys
     if comm is None:
         comm = COMM_WORLD
     if comm.Get_rank() == root:
-        print message
+        sys.stdout.write(str(message)+'\n')
+        sys.stdout.flush()
     return
