@@ -1,4 +1,4 @@
-/* $Id: config.h 16 2006-11-11 23:02:20Z dalcinl $ */
+/* $Id: config.h 53 2007-07-24 01:15:35Z dalcinl $ */
 
 #ifndef Py_MPI4PY_CONFIG_H
 #define Py_MPI4PY_CONFIG_H
@@ -780,6 +780,30 @@
 #endif
 #endif
 
+#ifndef HAVE_MPI_APPNUM
+#if HAVE_MPI_2 || defined(MPI_APPNUM)
+#define HAVE_MPI_APPNUM 1
+#else
+#define HAVE_MPI_APPNUM 0
+#endif
+#endif
+
+#ifndef HAVE_MPI_UNIVERSE_SIZE
+#if HAVE_MPI_2 || defined(MPI_UNIVERSE_SIZE)
+#define HAVE_MPI_UNIVERSE_SIZE 1
+#else
+#define HAVE_MPI_UNIVERSE_SIZE 0
+#endif
+#endif
+
+#ifndef HAVE_MPI_ERRCODES_IGNORE
+#if HAVE_MPI_2 || defined(MPI_ERRCODES_IGNORE)
+#define HAVE_MPI_ERRCODES_IGNORE 1
+#else
+#define HAVE_MPI_ERRCODES_IGNORE 0
+#endif
+#endif
+
 #ifndef HAVE_MPI_COMM_SPAWN
 #if HAVE_MPI_2
 #define HAVE_MPI_COMM_SPAWN 1
