@@ -1,22 +1,8 @@
-/* $Id: libmpi.h 24 2007-02-24 23:19:13Z dalcinl $ */
+/* $Id: libmpi.h 115 2007-07-31 23:20:54Z dalcinl $ */
 
 #ifndef Py_LIBMPI_H
 #define Py_LIBMPI_H
 
-/* Below, some stuff from the top of MPICH2 mpicxx.h ...
-
-   There is a name conflict between stdio.h and the MPI C++ binding 
-   with respect to the names SEEK_SET, SEEK_CUR, and SEEK_END.  MPI
-   wants these in the MPI namespace, but stdio.h will #define these
-   to integer values.  #undef'ing these can cause obscure problems
-   with other include files (such as iostream), so we instead use
-   #error to indicate a fatal error.  Users can either #undef 
-   the names before including mpi.h or include mpi.h *before* stdio.h
-   or iostream.
-
-   The MPI STANDARD HAS TO BE CHANGED to prevent this nonsense. 
-*/
-
 #include <Python.h>
 #include <mpi.h>
 
@@ -32,10 +18,17 @@ extern "C" {
 
 #endif /* !libmpi_MODULE */
 
+/* Base MPI Object Structure, 
+ * shared by all MPI types */
+typedef struct PyMPIObject {
+  PyObject_HEAD
+  PyObject* ob_mpi;
+} PyMPIObject;
+
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_INFO_NULL)
-#define libmpi_MPI_Info
+#define libmpi_defined_MPI_Info
 #define MPI_Info void*
 #define MPI_INFO_NULL ((MPI_Info)0)
 #endif
@@ -43,7 +36,7 @@ extern "C" {
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_WIN_NULL)
-#define libmpi_MPI_Win
+#define libmpi_defined_MPI_Win
 #define MPI_Win void*
 #define MPI_WIN_NULL ((MPI_Win)0)
 #endif
@@ -51,30 +44,34 @@ extern "C" {
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_FILE_NULL)
-#define libmpi_MPI_File
+#define libmpi_defined_MPI_File
 #define MPI_File void*
 #define MPI_FILE_NULL ((MPI_File)0)
 #endif
 #endif
 
+/* Inner Comm Object Structure
+ * holds a  MPI_Comm handle */
+
 typedef struct _Py_MPI_Comm {
   PyObject_HEAD
   MPI_Comm comm;
+  int isref;
 } _Py_MPI_Comm;
 
-typedef struct PyMPICommObject {
-  PyObject_HEAD
-  PyObject* comm;
-} PyMPICommObject;
+#define PyMPICommObject PyMPIObject
 
 #define PyMPIComm(ob) \
-  ((PyMPICommObject*)(ob))
+    ((PyMPICommObject*)(ob))
+
+#define PyMPIComm_Inner(ob) \
+    ((_Py_MPI_Comm*)(PyMPIComm(ob)->ob_mpi))
 
 #define PyMPIComm_AS_COMM(ob) \
-  (((_Py_MPI_Comm*)(PyMPIComm(ob)->comm))->comm)
+    (PyMPIComm_Inner(ob)->comm)
 
 #define PyMPIComm_AS_COMM_PTR(ob) \
-  (&PyMPIComm_AS_COMM(ob))
+    (&PyMPIComm_AS_COMM(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -124,24 +121,28 @@ static void (**PyMPIComm_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Datatype Object Structure
+ * holds a  MPI_Datatype handle */
+
 typedef struct _Py_MPI_Datatype {
   PyObject_HEAD
   MPI_Datatype datatype;
+  int isref;
 } _Py_MPI_Datatype;
 
-typedef struct PyMPIDatatypeObject {
-  PyObject_HEAD
-  PyObject* datatype;
-} PyMPIDatatypeObject;
+#define PyMPIDatatypeObject PyMPIObject
 
 #define PyMPIDatatype(ob) \
-  ((PyMPIDatatypeObject*)(ob))
+    ((PyMPIDatatypeObject*)(ob))
+
+#define PyMPIDatatype_Inner(ob) \
+    ((_Py_MPI_Datatype*)(PyMPIDatatype(ob)->ob_mpi))
 
 #define PyMPIDatatype_AS_DATATYPE(ob) \
-  (((_Py_MPI_Datatype*)(PyMPIDatatype(ob)->datatype))->datatype)
+    (PyMPIDatatype_Inner(ob)->datatype)
 
 #define PyMPIDatatype_AS_DATATYPE_PTR(ob) \
-  (&PyMPIDatatype_AS_DATATYPE(ob))
+    (&PyMPIDatatype_AS_DATATYPE(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -191,24 +192,28 @@ static void (**PyMPIDatatype_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Errhandler Object Structure
+ * holds a  MPI_Errhandler handle */
+
 typedef struct _Py_MPI_Errhandler {
   PyObject_HEAD
   MPI_Errhandler errhandler;
+  int isref;
 } _Py_MPI_Errhandler;
 
-typedef struct PyMPIErrhandlerObject {
-  PyObject_HEAD
-  PyObject* errhandler;
-} PyMPIErrhandlerObject;
+#define PyMPIErrhandlerObject PyMPIObject
 
 #define PyMPIErrhandler(ob) \
-  ((PyMPIErrhandlerObject*)(ob))
+    ((PyMPIErrhandlerObject*)(ob))
+
+#define PyMPIErrhandler_Inner(ob) \
+    ((_Py_MPI_Errhandler*)(PyMPIErrhandler(ob)->ob_mpi))
 
 #define PyMPIErrhandler_AS_ERRHANDLER(ob) \
-  (((_Py_MPI_Errhandler*)(PyMPIErrhandler(ob)->errhandler))->errhandler)
+    (PyMPIErrhandler_Inner(ob)->errhandler)
 
 #define PyMPIErrhandler_AS_ERRHANDLER_PTR(ob) \
-  (&PyMPIErrhandler_AS_ERRHANDLER(ob))
+    (&PyMPIErrhandler_AS_ERRHANDLER(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -258,24 +263,28 @@ static void (**PyMPIErrhandler_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner File Object Structure
+ * holds a  MPI_File handle */
+
 typedef struct _Py_MPI_File {
   PyObject_HEAD
   MPI_File file;
+  int isref;
 } _Py_MPI_File;
 
-typedef struct PyMPIFileObject {
-  PyObject_HEAD
-  PyObject* file;
-} PyMPIFileObject;
+#define PyMPIFileObject PyMPIObject
 
 #define PyMPIFile(ob) \
-  ((PyMPIFileObject*)(ob))
+    ((PyMPIFileObject*)(ob))
+
+#define PyMPIFile_Inner(ob) \
+    ((_Py_MPI_File*)(PyMPIFile(ob)->ob_mpi))
 
 #define PyMPIFile_AS_FILE(ob) \
-  (((_Py_MPI_File*)(PyMPIFile(ob)->file))->file)
+    (PyMPIFile_Inner(ob)->file)
 
 #define PyMPIFile_AS_FILE_PTR(ob) \
-  (&PyMPIFile_AS_FILE(ob))
+    (&PyMPIFile_AS_FILE(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -325,24 +334,28 @@ static void (**PyMPIFile_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Group Object Structure
+ * holds a  MPI_Group handle */
+
 typedef struct _Py_MPI_Group {
   PyObject_HEAD
   MPI_Group group;
+  int isref;
 } _Py_MPI_Group;
 
-typedef struct PyMPIGroupObject {
-  PyObject_HEAD
-  PyObject* group;
-} PyMPIGroupObject;
+#define PyMPIGroupObject PyMPIObject
 
 #define PyMPIGroup(ob) \
-  ((PyMPIGroupObject*)(ob))
+    ((PyMPIGroupObject*)(ob))
+
+#define PyMPIGroup_Inner(ob) \
+    ((_Py_MPI_Group*)(PyMPIGroup(ob)->ob_mpi))
 
 #define PyMPIGroup_AS_GROUP(ob) \
-  (((_Py_MPI_Group*)(PyMPIGroup(ob)->group))->group)
+    (PyMPIGroup_Inner(ob)->group)
 
 #define PyMPIGroup_AS_GROUP_PTR(ob) \
-  (&PyMPIGroup_AS_GROUP(ob))
+    (&PyMPIGroup_AS_GROUP(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -392,24 +405,28 @@ static void (**PyMPIGroup_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Info Object Structure
+ * holds a  MPI_Info handle */
+
 typedef struct _Py_MPI_Info {
   PyObject_HEAD
   MPI_Info info;
+  int isref;
 } _Py_MPI_Info;
 
-typedef struct PyMPIInfoObject {
-  PyObject_HEAD
-  PyObject* info;
-} PyMPIInfoObject;
+#define PyMPIInfoObject PyMPIObject
 
 #define PyMPIInfo(ob) \
-  ((PyMPIInfoObject*)(ob))
+    ((PyMPIInfoObject*)(ob))
+
+#define PyMPIInfo_Inner(ob) \
+    ((_Py_MPI_Info*)(PyMPIInfo(ob)->ob_mpi))
 
 #define PyMPIInfo_AS_INFO(ob) \
-  (((_Py_MPI_Info*)(PyMPIInfo(ob)->info))->info)
+    (PyMPIInfo_Inner(ob)->info)
 
 #define PyMPIInfo_AS_INFO_PTR(ob) \
-  (&PyMPIInfo_AS_INFO(ob))
+    (&PyMPIInfo_AS_INFO(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -459,24 +476,28 @@ static void (**PyMPIInfo_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Op Object Structure
+ * holds a  MPI_Op handle */
+
 typedef struct _Py_MPI_Op {
   PyObject_HEAD
   MPI_Op op;
+  int isref;
 } _Py_MPI_Op;
 
-typedef struct PyMPIOpObject {
-  PyObject_HEAD
-  PyObject* op;
-} PyMPIOpObject;
+#define PyMPIOpObject PyMPIObject
 
 #define PyMPIOp(ob) \
-  ((PyMPIOpObject*)(ob))
+    ((PyMPIOpObject*)(ob))
+
+#define PyMPIOp_Inner(ob) \
+    ((_Py_MPI_Op*)(PyMPIOp(ob)->ob_mpi))
 
 #define PyMPIOp_AS_OP(ob) \
-  (((_Py_MPI_Op*)(PyMPIOp(ob)->op))->op)
+    (PyMPIOp_Inner(ob)->op)
 
 #define PyMPIOp_AS_OP_PTR(ob) \
-  (&PyMPIOp_AS_OP(ob))
+    (&PyMPIOp_AS_OP(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -526,24 +547,28 @@ static void (**PyMPIOp_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Request Object Structure
+ * holds a  MPI_Request handle */
+
 typedef struct _Py_MPI_Request {
   PyObject_HEAD
   MPI_Request request;
+  int isref;
 } _Py_MPI_Request;
 
-typedef struct PyMPIRequestObject {
-  PyObject_HEAD
-  PyObject* request;
-} PyMPIRequestObject;
+#define PyMPIRequestObject PyMPIObject
 
 #define PyMPIRequest(ob) \
-  ((PyMPIRequestObject*)(ob))
+    ((PyMPIRequestObject*)(ob))
+
+#define PyMPIRequest_Inner(ob) \
+    ((_Py_MPI_Request*)(PyMPIRequest(ob)->ob_mpi))
 
 #define PyMPIRequest_AS_REQUEST(ob) \
-  (((_Py_MPI_Request*)(PyMPIRequest(ob)->request))->request)
+    (PyMPIRequest_Inner(ob)->request)
 
 #define PyMPIRequest_AS_REQUEST_PTR(ob) \
-  (&PyMPIRequest_AS_REQUEST(ob))
+    (&PyMPIRequest_AS_REQUEST(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -593,24 +618,28 @@ static void (**PyMPIRequest_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Status Object Structure
+ * holds a  MPI_Status handle */
+
 typedef struct _Py_MPI_Status {
   PyObject_HEAD
   MPI_Status status;
+  int isref;
 } _Py_MPI_Status;
 
-typedef struct PyMPIStatusObject {
-  PyObject_HEAD
-  PyObject* status;
-} PyMPIStatusObject;
+#define PyMPIStatusObject PyMPIObject
 
 #define PyMPIStatus(ob) \
-  ((PyMPIStatusObject*)(ob))
+    ((PyMPIStatusObject*)(ob))
+
+#define PyMPIStatus_Inner(ob) \
+    ((_Py_MPI_Status*)(PyMPIStatus(ob)->ob_mpi))
 
 #define PyMPIStatus_AS_STATUS(ob) \
-  (((_Py_MPI_Status*)(PyMPIStatus(ob)->status))->status)
+    (PyMPIStatus_Inner(ob)->status)
 
 #define PyMPIStatus_AS_STATUS_PTR(ob) \
-  (&PyMPIStatus_AS_STATUS(ob))
+    (&PyMPIStatus_AS_STATUS(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -660,24 +689,28 @@ static void (**PyMPIStatus_API)(void) = 0;
 
 #endif /* libmpi_MODULE */
 
+/* Inner Win Object Structure
+ * holds a  MPI_Win handle */
+
 typedef struct _Py_MPI_Win {
   PyObject_HEAD
   MPI_Win win;
+  int isref;
 } _Py_MPI_Win;
 
-typedef struct PyMPIWinObject {
-  PyObject_HEAD
-  PyObject* win;
-} PyMPIWinObject;
+#define PyMPIWinObject PyMPIObject
 
 #define PyMPIWin(ob) \
-  ((PyMPIWinObject*)(ob))
+    ((PyMPIWinObject*)(ob))
+
+#define PyMPIWin_Inner(ob) \
+    ((_Py_MPI_Win*)(PyMPIWin(ob)->ob_mpi))
 
 #define PyMPIWin_AS_WIN(ob) \
-  (((_Py_MPI_Win*)(PyMPIWin(ob)->win))->win)
+    (PyMPIWin_Inner(ob)->win)
 
 #define PyMPIWin_AS_WIN_PTR(ob) \
-  (&PyMPIWin_AS_WIN(ob))
+    (&PyMPIWin_AS_WIN(ob))
 
 
 #if defined(libmpi_MODULE)
@@ -728,20 +761,20 @@ static void (**PyMPIWin_API)(void) = 0;
 #endif /* libmpi_MODULE */
 
 
-#if defined(libmpi_MPI_Info)
-#undef libmpi_MPI_Info
+#if defined(libmpi_defined_MPI_Info)
+#undef libmpi_defined_MPI_Info
 #undef MPI_Info
 #undef MPI_INFO_NULL
 #endif
 
-#if defined(libmpi_MPI_Win)
-#undef libmpi_MPI_Win
+#if defined(libmpi_defined_MPI_Win)
+#undef libmpi_defined_MPI_Win
 #undef MPI_Win
 #undef MPI_WIN_NULL
 #endif
 
-#if defined(libmpi_MPI_File)
-#undef libmpi_MPI_File
+#if defined(libmpi_defined_MPI_File)
+#undef libmpi_defined_MPI_File
 #undef MPI_File
 #undef MPI_FILE_NULL
 #endif
