@@ -2,7 +2,7 @@
 
 # $Id$
 
-import sys
+import sys, os
 from os.path import join, extsep, pardir
 import re
 
@@ -10,6 +10,7 @@ if len(sys.argv) > 1 and  sys.argv[1] == '-o':
     outdir = sys.argv[2]
 else:
     outdir = pardir
+assert os.path.isdir(outdir)
 
 mpiobjs = ['op',
            'win',
@@ -25,8 +26,8 @@ mpiobjs = ['op',
 mpiobjs.sort()
 
 fname,h,c  = 'object','h','c'
-header_code = file(fname+extsep+h).read()
-source_code = file(fname+extsep+c).read()
+header_code = open(fname+extsep+h).read()
+source_code = open(fname+extsep+c).read()
 
 header_code = re.sub(r"<(\w+)>", r"%(\1)s", header_code)
 source_code = re.sub(r"<(\w+)>", r"%(\1)s", source_code)
@@ -73,13 +74,13 @@ object_c = '\n'.join(object_c)
 
 libmpi, h, c  = 'libmpi', 'h', 'c'
 
-libmpi_h = file(join(libmpi+extsep+h)).read()
-libmpi_c = file(join(libmpi+extsep+c)).read()
+libmpi_h = open(join(libmpi+extsep+h)).read()
+libmpi_c = open(join(libmpi+extsep+c)).read()
 
 libmpi_h = libmpi_h.replace('<MPI_OBJECT_H>', object_h)
 libmpi_c = libmpi_c.replace('<MPI_OBJECT_C>', object_c)
 
-file(join(outdir, libmpi+extsep+h),'w').write(libmpi_h)
-file(join(outdir, libmpi+extsep+c),'w').write(libmpi_c)
+open(join(outdir, libmpi+extsep+h),'w').write(libmpi_h)
+open(join(outdir, libmpi+extsep+c),'w').write(libmpi_c)
 
 # --------------------------------------------------------------------
