@@ -3,20 +3,6 @@
 #ifndef Py_LIBMPI_H
 #define Py_LIBMPI_H
 
-/* Below, some stuff from the top of MPICH2 mpicxx.h ...
-
-   There is a name conflict between stdio.h and the MPI C++ binding 
-   with respect to the names SEEK_SET, SEEK_CUR, and SEEK_END.  MPI
-   wants these in the MPI namespace, but stdio.h will #define these
-   to integer values.  #undef'ing these can cause obscure problems
-   with other include files (such as iostream), so we instead use
-   #error to indicate a fatal error.  Users can either #undef 
-   the names before including mpi.h or include mpi.h *before* stdio.h
-   or iostream.
-
-   The MPI STANDARD HAS TO BE CHANGED to prevent this nonsense. 
-*/
-
 #include <Python.h>
 #include <mpi.h>
 
@@ -32,10 +18,17 @@ extern "C" {
 
 #endif /* !libmpi_MODULE */
 
+/* Base MPI Object Structure, 
+ * shared by all MPI types */
+typedef struct PyMPIObject {
+  PyObject_HEAD
+  PyObject* ob_mpi;
+} PyMPIObject;
+
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_INFO_NULL)
-#define libmpi_MPI_Info
+#define libmpi_defined_MPI_Info
 #define MPI_Info void*
 #define MPI_INFO_NULL ((MPI_Info)0)
 #endif
@@ -43,7 +36,7 @@ extern "C" {
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_WIN_NULL)
-#define libmpi_MPI_Win
+#define libmpi_defined_MPI_Win
 #define MPI_Win void*
 #define MPI_WIN_NULL ((MPI_Win)0)
 #endif
@@ -51,7 +44,7 @@ extern "C" {
 
 #if !defined(MPI_VERSION) || (MPI_VERSION < 2)
 #if !defined(MPI_FILE_NULL)
-#define libmpi_MPI_File
+#define libmpi_defined_MPI_File
 #define MPI_File void*
 #define MPI_FILE_NULL ((MPI_File)0)
 #endif
@@ -59,20 +52,20 @@ extern "C" {
 
 <MPI_OBJECT_H>
 
-#if defined(libmpi_MPI_Info)
-#undef libmpi_MPI_Info
+#if defined(libmpi_defined_MPI_Info)
+#undef libmpi_defined_MPI_Info
 #undef MPI_Info
 #undef MPI_INFO_NULL
 #endif
 
-#if defined(libmpi_MPI_Win)
-#undef libmpi_MPI_Win
+#if defined(libmpi_defined_MPI_Win)
+#undef libmpi_defined_MPI_Win
 #undef MPI_Win
 #undef MPI_WIN_NULL
 #endif
 
-#if defined(libmpi_MPI_File)
-#undef libmpi_MPI_File
+#if defined(libmpi_defined_MPI_File)
+#undef libmpi_defined_MPI_File
 #undef MPI_File
 #undef MPI_FILE_NULL
 #endif
