@@ -1,8 +1,11 @@
 # Author:    Lisandro Dalcin
 # Contact:   dalcinl@gmail.com
-# Revision:  $Revision$
-# Date:      $Date$
 # Copyright: This module has been placed in the public domain.
+# Id:        $Id: mpidistutils.py 106 2007-07-31 18:11:02Z dalcinl $
+
+"""
+Support for building mpi4py with distutils.
+"""
 
 # --------------------------------------------------------
 
@@ -25,40 +28,45 @@ from distutils import log
 
 
 def customize_compiler(compiler, environ=None):
-    """Do any platform-specific customization of a CCompiler instance.
+    """
+    Do any platform-specific customization of a CCompiler instance.
 
     Mainly needed on Unix, so we can plug in the information that
     varies across Unices and is stored in Python's Makefile.
     """
+    from distutils.sysconfig import get_config_vars
     if environ is None:
         environ = os.environ
+        
     if compiler.compiler_type == 'unix':
-        (cc, cxx, opt, basecflags, ccshared, ldshared, so_ext) = \
-            sysconfig.get_config_vars('CC', 'CXX', 'OPT', 'BASECFLAGS', 'CCSHARED', 'LDSHARED', 'SO')
+        (cc, cxx, opt,
+         basecflags, cflags, ccshared,
+         ldshared, so_ext) = \
+            get_config_vars('CC', 'CXX',
+                            'BASECFLAGS', 'OPT', 'CFLAGS',
+                            'CCSHARED', 'LDSHARED', 'SO')
 
-        if environ.has_key('CC'):
+        if 'CC' in environ:
             cc = environ['CC']
-        if environ.has_key('CXX'):
+        if 'CXX' in environ:
             cxx = environ['CXX']
-        if environ.has_key('LDSHARED'):
+        if 'LDSHARED' in environ:
             ldshared = environ['LDSHARED']
-        if environ.has_key('CPP'):
+        if 'CPP' in environ:
             cpp = environ['CPP']
         else:
             cpp = cc + " -E"           # not always
-        if environ.has_key('LDFLAGS'):
+        if 'LDFLAGS' in environ:
             ldshared = ldshared + ' ' + environ['LDFLAGS']
-        if basecflags:
-            opt = basecflags + ' ' + opt
-        if environ.has_key('CFLAGS'):
-            opt = opt + ' ' + environ['CFLAGS']
+        if 'CFLAGS' in environ:
+            cflags = basecflags + ' ' + opt + ' ' + environ['CFLAGS']
             ldshared = ldshared + ' ' + environ['CFLAGS']
-        if environ.has_key('CPPFLAGS'):
+        if 'CPPFLAGS' in environ:
             cpp = cpp + ' ' + environ['CPPFLAGS']
-            opt = opt + ' ' + environ['CPPFLAGS']
+            cflags = cflags + ' ' + environ['CPPFLAGS']
             ldshared = ldshared + ' ' + environ['CPPFLAGS']
 
-        cc_cmd = cc + ' ' + opt
+        cc_cmd = cc + ' ' + cflags
         compiler.set_executables(
             preprocessor=cpp,
             compiler=cc_cmd,
@@ -70,6 +78,9 @@ def customize_compiler(compiler, environ=None):
         compiler.shared_lib_extension = so_ext
 
 def customize_mpi_environ(mpicc, mpicxx=None, environ=None):
+    """
+    Replace normal compilers with MPI compilers
+    """
     if environ is None:
         environ = dict(os.environ)
     if mpicc: # C compiler
@@ -88,6 +99,9 @@ def customize_mpi_environ(mpicc, mpicxx=None, environ=None):
     return environ
 
 def _find_mpi_compiler(envvars, executables, path=None):
+    """
+    Find MPI compilers in environment and path.
+    """
     # search in environment
     if envvars:
         if isinstance(envvars, str):
@@ -155,8 +169,6 @@ def _config_parser(section, filenames, raw=False, vars=None):
                 macro = (m, None)
             macros.append(macro)
         config_info['define_macros'] = macros
-    #from pprint import pprint
-    #pprint(config_info)
     return config_info
 
 
@@ -189,7 +201,7 @@ def _find_mpi_config(section, filenames,
 
 def _configure(extension, confdict):
     if confdict is None: return
-    for key, value in confdict.iteritems():
+    for key, value in confdict.items():
         if hasattr(extension, key):
             item = getattr(extension, key)
             if type(item) is list:
@@ -211,6 +223,7 @@ from distutils.command import build_ext as cmd_build_ext
 from distutils.command import sdist as cmd_sdist
 
 from distutils.errors import DistutilsPlatformError
+from distutils.errors import DistutilsOptionError
 
 
 ConfigTests = [
@@ -355,6 +368,12 @@ ConfigTests = [
      'MPI_Comm_connect((char*)0,MPI_INFO_NULL,0,MPI_COMM_NULL,(MPI_Comm*)0);'),
     ('MPI_COMM_DISCONNECT',
      'MPI_Comm comm = MPI_COMM_NULL; MPI_Comm_disconnect(&comm);'),
+    ('MPI_APPNUM',
+     'int keyval = MPI_KEYVAL_INVALID; keyval = MPI_APPNUM;'),
+    ('MPI_UNIVERSE_SIZE',
+     'int keyval = MPI_KEYVAL_INVALID; keyval = MPI_UNIVERSE_SIZE;'),
+    ('MPI_ERRCODES_IGNORE',
+     'int* ec; ec = MPI_ERRCODES_IGNORE;'),
     ('MPI_COMM_SPAWN',
      'MPI_Comm_spawn((char*)0,(char**)0,0,' \
      'MPI_INFO_NULL,0,MPI_COMM_NULL,(MPI_Comm*)0,(int*)0);'),
@@ -394,7 +413,7 @@ int main(int argc, char **argv) {
 """
 
 
-mpi_cmd_opts = [
+cmd_mpi_opts = [
 
     ('mpicc=',   None,
      "MPI C compiler command, "
@@ -420,27 +439,62 @@ mpi_cmd_opts = [
     
     ('try-mpi-2', None,
      "test for availability of MPI 2 features"),
+
+    ('thread-level=', None,
+     "initialize MPI with support for threads"),
+
+    ('enable-fast-errcheck', None,
+     "faster error checking (by forcing  MPI_ERRORS_RETURN)"),
+
+    ('enable-leak-warning', None,
+     "isuue a warning if any MPI handle is (possibly) leaked"),
+    
     ]
 
+def _cmd_opts_names(cmd_opts):
+    optlist = []
+    for (option, _, _) in cmd_opts:
+        if option[-1] == "=":
+            option = option[:-1]
+        option = option.replace('-','_')
+        optlist.append(option)
+    return optlist
+        
+def _thread_level(thread_level):
+    if not thread_level:
+        return None
+    thread_level = thread_level.lower()
+    if thread_level == 'none':
+        return None
+    valid_levels = ['single', 'funneled', 'serialized', 'multiple']
+    if thread_level not in valid_levels:
+        raise DistutilsOptionError, \
+              ("thread-level must be one of " + ','.join(valid_levels))
+    return thread_level
+
+def _enable_fast_errcheck(flag):
+    if flag:
+        return 1
+    else:
+        return 0
 
 
 class config(cmd_config.config):
 
-    user_options = cmd_config.config.user_options + mpi_cmd_opts
+    user_options = cmd_config.config.user_options + cmd_mpi_opts
 
     def initialize_options(self):
         cmd_config.config.initialize_options(self)
         self.noisy = 0
-        self.mpicc = None
-        self.mpicxx = None
-        self.mpi = None
-        self.mpi_cfg = None
-        self.try_mpi_2 = None
+        mpiopts = _cmd_opts_names(cmd_mpi_opts)
+        for op in mpiopts:
+            setattr(self, op, None)
         
     def finalize_options(self):
         cmd_config.config.finalize_options(self)
         if not self.noisy:
             self.dump_source = 0
+        self.thread_level = _thread_level(self.thread_level)
         
     def find_mpi_compiler(self, envvars, executables, path=None):
         return _find_mpi_compiler(envvars, executables, path)
@@ -504,42 +558,37 @@ class config(cmd_config.config):
 
 class build(cmd_build.build):
 
-    user_options = cmd_build.build.user_options + mpi_cmd_opts
+    user_options = cmd_build.build.user_options + cmd_mpi_opts
     
     def initialize_options(self):
         cmd_build.build.initialize_options(self)
-        self.mpicc = None
-        self.mpicxx = None
-        self.mpi = None
-        self.mpi_cfg = None
-        self.try_mpi_2 = None
+        mpiopts = _cmd_opts_names(cmd_mpi_opts)
+        for op in mpiopts:
+            setattr(self, op, None)
         
     def finalize_options(self):
         cmd_build.build.finalize_options(self)
         config_cmd = self.get_finalized_command('config')
         if isinstance(config_cmd,  config):
-            self.set_undefined_options('config',
-                                       ('mpicc',    'mpicc'),
-                                       ('mpicxx',   'mpicxx'),
-                                       ('mpi',      'mpi'),
-                                       ('mpi_cfg',  'mpi_cfg'),
-                                       ('try_mpi_2','try_mpi_2'))
-        
+            mpiopts = _cmd_opts_names(cmd_mpi_opts)
+            optlist = tuple(zip(mpiopts, mpiopts))
+            self.set_undefined_options('config', *optlist)
+        self.thread_level = _thread_level(self.thread_level)
+
 
 class build_ext(cmd_build_ext.build_ext):
 
-    user_options = cmd_build_ext.build_ext.user_options + mpi_cmd_opts
+    user_options = cmd_build_ext.build_ext.user_options + cmd_mpi_opts
 
     def initialize_options(self):
         cmd_build_ext.build_ext.initialize_options(self)
-        self.mpicc = None
-        self.mpicxx = None
-        self.mpi = None
-        self.mpi_cfg = None
-        self.try_mpi_2 = None
+        mpiopts = _cmd_opts_names(cmd_mpi_opts)
+        for op in mpiopts:
+            setattr(self, op, None)
 
     def finalize_options(self):
         cmd_build_ext.build_ext.finalize_options(self)
+        # XXX I have to figure out why this is needed
         import sys, os
         if (sys.platform.startswith('linux') and
             sysconfig.get_config_var('Py_ENABLE_SHARED')):
@@ -553,15 +602,12 @@ class build_ext(cmd_build_ext.build_ext):
                 pass
             pylib_dir = os.path.join(sys.prefix, "lib")
             self.library_dirs.append(pylib_dir)
-                
         build_cmd = self.get_finalized_command('build')
         if isinstance(build_cmd,  build):
-            self.set_undefined_options('build',
-                                       ('mpicc',    'mpicc'),
-                                       ('mpicxx',   'mpicxx'),
-                                       ('mpi',      'mpi'),
-                                       ('mpi_cfg',  'mpi_cfg'),
-                                       ('try_mpi_2','try_mpi_2'))
+            mpiopts = _cmd_opts_names(cmd_mpi_opts)
+            optlist = tuple(zip(mpiopts, mpiopts))
+            self.set_undefined_options('build', *optlist)
+        self.thread_level = _thread_level(self.thread_level)
 
     def build_extensions(self):
         # First, sanity-check the 'extensions' list
@@ -581,7 +627,25 @@ class build_ext(cmd_build_ext.build_ext):
             macros = get_macros(ConfigTests, compiler, config_info)
             for (name, value) in macros:
                 self.compiler.define_macro(name, value)
-                log.info('%s %d' % (name, value))
+                log.info('#define %s %d' % (name, value))
+        # extra configuration, MPI thread level support
+        if self.thread_level:
+            levels = ['single', 'funneled', 'serialized', 'multiple']
+            name = 'MPI4PY_THREAD_LEVEL'
+            value = levels.index(self.thread_level)
+            self.compiler.define_macro(name, value)
+            log.info('#define %s %d' % (name, value))
+        # extra configuration, leak warnings
+        if self.enable_leak_warning:
+            name, value = 'MPI4PY_ENABLE_LEAK_WARNING', 1
+            self.compiler.define_macro(name, value)
+            log.info('#define %s %d' % (name, value))
+        # extra configuration, fast error checking
+        if self.enable_fast_errcheck:
+            name, value = 'MPI4PY_ENABLE_FAST_ERRCHECK', 1
+            self.compiler.define_macro(name, value)
+            log.info('#define %s %d' % (name, value))
+
         # build extensions
         for ext in self.extensions:
             self.build_extension(ext)
