@@ -1,8 +1,10 @@
 #! /usr/bin/env python
 
 from mpi4py import MPI
+import sys
 
-child = MPI.COMM_WORLD.Spawn('child.py', [], MPI.COMM_WORLD.size+1, MPI.INFO_NULL, root=0)
+child = MPI.COMM_WORLD.Spawn(sys.executable, ['child.py'],
+                             MPI.COMM_WORLD.size+1, MPI.INFO_NULL, root=0)
 assert child != MPI.COMM_NULL
 
 print 'parent size: %d, child size: %d' %(child.size, child.remote_size)
