@@ -19,9 +19,9 @@ coverage.exclude('class Win')
 coverage.exclude('_mpi_init')
 coverage.exclude('_mpi_finalize')
 coverage.exclude('SWIG')
-coverage.exclude('distribute')
-coverage.exclude('pprint')
-coverage.exclude('rprint')
+coverage.exclude('_distribute')
+coverage.exclude('_pprint')
+coverage.exclude('_rprint')
 
 import glob
 import unittest
