@@ -1,13 +1,15 @@
-import unittest, tempfile
+import unittest, tempfile, os
 from mpi4py import MPI
 
 class TestFileBase(object):
 
     COMM = MPI.COMM_NULL
 
+    prefix = 'mpi4py'
+    
     def setUp(self):
-        self.FILE = None
-        self.fname = tempfile.mktemp(prefix='mpi4py')
+        self.FILE = MPI.FILE_NULL
+        self.fd, self.fname = tempfile.mkstemp(prefix=self.prefix)
         self.amode = MPI.MODE_RDWR | MPI.MODE_CREATE
         #self.amode |= MPI.MODE_DELETE_ON_CLOSE
         try:
@@ -15,15 +17,17 @@ class TestFileBase(object):
                                       self.fname, self.amode,
                                       MPI.INFO_NULL)
         except NotImplementedError:
+            os.close(self.fd)
             return
         
     def tearDown(self):
-        if self.FILE is None: return
         if self.FILE == MPI.FILE_NULL: return
         amode = self.FILE.amode
         self.FILE.Close()
         if not (amode & MPI.MODE_DELETE_ON_CLOSE):
             MPI.File.Delete(self.fname, MPI.INFO_NULL)
+        os.close(self.fd)
+        
 
     def testPreallocate(self):
         self.FILE.Preallocate(0)
@@ -42,19 +46,26 @@ class TestFileBase(object):
     def testGetSetSize(self):
         size = self.FILE.Get_size()
         self.assertEqual(size, 0)
+        size = self.FILE.size
+        self.assertEqual(size, 0)
         self.FILE.Set_size(100)
         size = self.FILE.Get_size()
         self.assertEqual(size, 100)
+        size = self.FILE.size
+        self.assertEqual(size, 100)
 
     def testGetGroup(self):
         fgroup = self.FILE.Get_group()
         cgroup = self.COMM.Get_group()
         gcomp = MPI.Group.Compare(fgroup, cgroup)
         self.assertEqual(gcomp, MPI.IDENT)
+        fgroup.Free()
+        cgroup.Free()
 
     def testGetAmode(self):
         amode = self.FILE.Get_amode()
         self.assertEqual(self.amode, amode)
+        self.assertEqual(self.FILE.amode, self.amode)
 
     def testGetSetInfo(self):
         info = self.FILE.Get_info()
@@ -78,6 +89,10 @@ class TestFileBase(object):
                     self.assertEqual(etype,   et)
                     self.assertEqual(ftype,   ft)
                     self.assertEqual(datarep, dr)
+                    #try: et.Free()
+                    #except MPI.Exception: pass
+                    #try: ft.Free()
+                    #except MPI.Exception: pass
 
     def testGetSetAtomicity(self):
         atom = self.FILE.Get_atomicity()
@@ -107,7 +122,7 @@ class TestFileBase(object):
         self.assertEqual(pos, offset)
 
     def testGetByteOffset(self):
-        for offset in xrange(10):
+        for offset in range(10):
             disp = self.FILE.Get_byte_offset(offset)
             self.assertEqual(disp, offset)
 
@@ -115,13 +130,10 @@ class TestFileBase(object):
         extent = self.FILE.Get_type_extent(MPI.BYTE)
         self.assertEqual(extent, 1)
 
-    def testErrhandler(self):
+    def testGetErrhandler(self):
         eh = self.FILE.Get_errhandler()
         self.assertEqual(eh, MPI.ERRORS_RETURN)
-
-    def testProperties(self):
-        self.assertEqual(self.FILE.amode, self.amode)
-        self.assertEqual(self.FILE.size,  0)
+        eh.Free()
 
 class TestFileNull(unittest.TestCase):
 
@@ -131,22 +143,45 @@ class TestFileNull(unittest.TestCase):
     def testGetSetErrhandler(self):
         eh = MPI.FILE_NULL.Get_errhandler()
         self.assertEqual(eh, MPI.ERRORS_RETURN)
+        eh.Free()
         MPI.FILE_NULL.Set_errhandler(MPI.ERRORS_ARE_FATAL)
         eh = MPI.FILE_NULL.Get_errhandler()
         self.assertEqual(eh, MPI.ERRORS_ARE_FATAL)
+        eh.Free()
         MPI.FILE_NULL.Set_errhandler(MPI.ERRORS_RETURN)
         eh = MPI.FILE_NULL.Get_errhandler()
         self.assertEqual(eh, MPI.ERRORS_RETURN)
+        eh.Free()
         
 
 class TestFileSelf(TestFileBase, unittest.TestCase):
     COMM = MPI.COMM_SELF
-    
+    prefix = TestFileBase.prefix + ('-%d' % MPI.COMM_WORLD.Get_rank())
 
 
+_name, _version = MPI._mpi_info()
+if _name == 'OpenMPI':
+    if MPI.Query_thread() > MPI.THREAD_SINGLE:
+        del TestFileBase.testPreallocate
+        del TestFileBase.testGetSetInfo
+        del TestFileBase.testGetSetAtomicity
+        del TestFileBase.testSync
+        del TestFileBase.testGetSetSize
+        del TestFileBase.testGetSetView
+        del TestFileBase.testGetByteOffset
+        del TestFileBase.testGetTypeExtent
+        del TestFileBase.testSeekGetPosition
+        del TestFileBase.testSeekGetPositionShared
+if _name == 'LAM/MPI':
+    del TestFileNull
+    del TestFileSelf
+if _name == 'MPICH':
+    del TestFileNull
+    del TestFileSelf
+
 if __name__ == '__main__':
-    #unittest.main()
     try:
         unittest.main()
     except SystemExit:
         pass
+    
