@@ -70,8 +70,8 @@ class TestP2PBufBase(object):
         source = (rank - 1) % size
         for mkbufs, array, equal in arrayimpl:
             for mkbuf in mkbufs:
-                for typecode, datatype in typemap.iteritems():
-                    for s in xrange(0, size):
+                for typecode, datatype in typemap.items():
+                    for s in range(0, size):
                         sbuf = array(typecode, [s] * s)
                         rbuf = array(typecode, [-1] * s)
                         self.COMM.Sendrecv(mkbuf(sbuf, datatype), dest, 0,
@@ -84,8 +84,8 @@ class TestP2PBufBase(object):
         rank = self.COMM.Get_rank()
         for mkbufs, array, equal in arrayimpl:
             for mkbuf in mkbufs:
-                for typecode, datatype in typemap.iteritems():
-                    for s in xrange(0, size):
+                for typecode, datatype in typemap.items():
+                    for s in range(0, size):
                         sbuf = array(typecode, [s] * s)
                         rbuf = array(typecode, [-1] * s)
                         mem  = array(typecode, [0] * (s+MPI.BSEND_OVERHEAD))
@@ -125,8 +125,8 @@ class TestP2PBufBase(object):
         source = (rank - 1) % size
         for mkbufs, array, equal in arrayimpl:
             for mkbuf in mkbufs:
-                for typecode, datatype in typemap.iteritems():
-                    for s in xrange(size):
+                for typecode, datatype in typemap.items():
+                    for s in range(size):
                         #
                         sbuf = array(typecode, [s]*s)
                         rbuf = array(typecode, [-1]*s)
@@ -168,7 +168,6 @@ class TestP2PBufWorld(TestP2PBufBase, unittest.TestCase):
 
 
 if __name__ == '__main__':
-    #unittest.main()
     try:
         unittest.main()
     except SystemExit:
