@@ -1,13 +1,23 @@
-import unittest
+import sys, unittest
 from mpi4py import MPI
 
 # MPI.Comm.SERIALIZER = MPI.Pickle(protocol=0)
 # MPI.Comm.SERIALIZER = MPI.Marshal
 
-messages = [None, 7, 3.14, 1+2j, 'mpi4py',
-            [None, 7, 3.14, 1+2j, 'mpi4py'],
-            (None, 7, 3.14, 1+2j, 'mpi4py'),
-            dict(k1=None,k2=7,k3=3.14,k4=1+2j,k5='mpi4py')]
+_basic = [None,
+          True, False,
+          -7, 0, 7, sys.maxint,
+          -2.17, 0.0, 3.14,
+          1+2j, 2-3j,
+          'mpi4py',
+          ]
+messages = _basic
+messages += [ list(_basic),
+              tuple(_basic),
+              dict([('k%d' % key, val)
+                    for key, val in enumerate(_basic)])
+              ]
+
 
 class TestP2PObjBase(object):
     
@@ -86,7 +96,6 @@ class TestP2PObjWorldDup(TestP2PObjBase, unittest.TestCase):
 
 
 if __name__ == '__main__':
-    #unittest.main()
     try:
         unittest.main()
     except SystemExit:
